@@ -216,13 +216,17 @@ History                 no X-KDE-PluginKeyword in kcmhistory.desktop
     list = KServiceTypeTrader::self()->query( QStringLiteral("KCModule"), QStringLiteral("[X-KDE-ParentApp] == 'kinfocenter'") );
   }
 
+  bool no_children_present = true;
+
   for ( KService::List::const_iterator it = list.constBegin(); it != list.constEnd(); ++it )
   {
     KService::Ptr s(*it);
     QString desktopFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kservices5/") + s->entryPath() );
     createItemFromDesktopFile( topItem, desktopFile );
+    no_children_present = false;
     }
     topItem->sortChildren( 0, Qt::AscendingOrder /* ascending */ );
+    topItem->setHidden(no_children_present);
 }
 
 void Navigator::insertIOSlaveDocs( const QString &name, NavigatorItem *topItem )
@@ -275,7 +279,7 @@ void Navigator::insertInfoDocs( NavigatorItem *topItem )
 void Navigator::insertScrollKeeperDocs( NavigatorItem *topItem )
 {
   ScrollKeeperTreeBuilder *builder = new ScrollKeeperTreeBuilder( this );
-  builder->build( topItem );
+  builder->buildOrHide( topItem );
 }
 
 void Navigator::selectItem( const QUrl &url )
@@ -291,10 +295,12 @@ void Navigator::selectItem( const QUrl &url )
   // Make sure that we match both the original URL as well as
   // its counterpart.
   QUrl alternativeURL = url;
+  QUrl contentsItemURL = url;
   if (url.hasFragment())
   {
      alternativeURL.setQuery(QStringLiteral("anchor=")+url.fragment());
      alternativeURL.setFragment(QString());
+     contentsItemURL.setFragment(QString());
   }
 
   // If the navigator already has the given URL selected, do nothing.
@@ -319,22 +325,32 @@ void Navigator::selectItem( const QUrl &url )
     }
   }
   
+  NavigatorItem *contentsItem = nullptr;
   QTreeWidgetItemIterator it( mContentsTree );
   while ( (*it) ) {
     NavigatorItem *item = static_cast<NavigatorItem *>( (*it) );
     QUrl itemUrl( item->entry()->url() );
     if ( (itemUrl == url) || (itemUrl == alternativeURL) ) {
-      mContentsTree->setCurrentItem( item );
       // If the current item was not selected and remained unchanged it
       // needs to be explicitly selected
       mContentsTree->setCurrentItem(item);
       item->setExpanded( true );
       break;
     }
+    if ( (contentsItem == nullptr) && (itemUrl == contentsItemURL) ) {
+      contentsItem = item;
+    }
     ++it;
   }
   if ( !(*it) ) {
-    clearSelection();
+    // if search with fragment didn't find anything, but item without fragment was found, use it
+    if ( contentsItem != nullptr ) {
+      mContentsTree->setCurrentItem(contentsItem);
+      item->setExpanded( true );
+      mSelected = true;
+    } else {
+      clearSelection();
+    }
   } else {
     mSelected = true;
   }
